
#include <errno.h>
#include <fcntl.h>
#include <sys/vfs.h>
#include <filesys/fatfs/ff.h>


vfs_file_t *
_sdsd_open(int pclk, int pcmd, int pdat0, int ppwr, int pled)
{
    int r;
    int drv = 0;
    struct __using("blkdrvr/sdsd.cc") *SDSD;
    unsigned long long pmask;
    vfs_file_t *handle;

    SDSD = _gc_alloc_managed(sizeof(*SDSD));

#ifdef _DEBUG
    __builtin_printf("sdsd_open: using pins: %d %d %d %d %d\n", pclk, pcmd, pdat0, ppwr, pled);
#endif
    pmask = (1ULL << pclk) | (1ULL << pcmd) | (7ULL << pdat0);
    if( ppwr >= 0 )    // optional SD slot power switch
        pmask |= 1ULL << ppwr;
    if( pled >= 0 )    // optional SD slot activity LED
        pmask |= 1ULL << pled;
    if (!_usepins(pmask)) {
        _gc_free(SDSD);
        _seterror(EBUSY);
        return 0;
    }
    SDSD->f_pinmask = pmask;
    r = SDSD->disk_setpins(drv, pclk, pcmd, pdat0, ppwr, pled);
    if (r == 0)
        r = SDSD->disk_initialize(0);
    if (r != 0) {
#ifdef _DEBUG
       __builtin_printf("sd card initialize: result=[%d]\n", r);
       _waitms(1000);
#endif
       goto cleanup_and_out;
    }
    handle = _get_vfs_file_handle();
    if (!handle) goto cleanup_and_out;

    handle->flags = O_RDWR;
    handle->bufmode = _IONBF;
    handle->state = _VFS_STATE_INUSE | _VFS_STATE_WROK | _VFS_STATE_RDOK;
    handle->read = &SDSD->v_read;
    handle->write = &SDSD->v_write;
    handle->close = &SDSD->v_close;
    handle->ioctl = &SDSD->v_ioctl;
    handle->flush = &SDSD->v_flush;
    handle->lseek = &SDSD->v_lseek;
    handle->putcf = &SDSD->v_putc;
    handle->getcf = &SDSD->v_getc;
    return handle;
cleanup_and_out:
    _freepins(pmask);
    _gc_free(SDSD);
    _seterror(EIO);
    return 0;
}

